@@ -1,8 +1,10 @@
 import datetime
 import logging
 
+from django.conf import settings
 from django.contrib.auth.models import User
 from django.contrib.gis.db import models
+from django.core.cache import cache
 from jsonfield.fields import JSONField
 
 logger = logging.getLogger('location.models')
@@ -25,6 +27,13 @@ except ImportError:
     Neighborhood = None
 
 
+CACHE_PREFIX = getattr(
+    settings,
+    'DJANGO_LOCATION_CACHE_PREFIX',
+    'LOCATION',
+)
+
+
 class LocationSourceType(models.Model):
     name = models.CharField(max_length=255)
     icon = models.ImageField(
@@ -89,34 +98,68 @@ class LocationSnapshot(models.Model):
 
     objects = models.GeoManager()
 
+    def get_cache_key(self, name):
+        return '%s:%s:%s:%s' % (
+            CACHE_PREFIX,
+            self.__class__.__name__,
+            self.pk,
+            name
+        )
+
+    def get_cached(self, name, cls):
+        pk = cache.get(
+            self.get_cache_key(name)
+        )
+        if pk:
+            return cls.objects.get(pk=pk)
+        return None
+
+    def set_cached(self, name, value):
+        cache.set(
+            self.get_cache_key(name),
+            value.pk
+        )
+
     @property
     def city(self):
         if PlaceBoundary:
-            if not hasattr(self, '_city'):
-                try:
-                    self._city = PlaceBoundary.get_containing(self.location)
-                except PlaceBoundary.DoesNotExist:
-                    self._city = None
-            return self._city
+            cached = self.get_cached('city', PlaceBoundary)
+            if cached:
+                return cached
+            try:
+                result = PlaceBoundary.get_containing(self.location)
+                self.set_cached('city', result)
+                return result
+            except PlaceBoundary.DoesNotExist:
+                pass
+        return None
 
     @property
     def neighborhood(self):
         if Neighborhood:
-            if not hasattr(self, '_neighborhood'):
-                try:
-                    self._neighborhood = Neighborhood.get_containing(self.location)
-                except Neighborhood.DoesNotExist:
-                    self._neighborhood = None
-            return self._neighborhood
+            cached = self.get_cached('neighborhood', Neighborhood)
+            if cached:
+                return cached
+            try:
+                result = Neighborhood.get_containing(self.location)
+                self.set_cached('neighborhood', result)
+                return result
+            except Neighborhood.DoesNotExist:
+                pass
+        return None
 
     def find_nearest_city(self):
         if PlaceBoundary:
-            if not hasattr(self, '_nearest_city'):
-                try:
-                    self._nearest_city = PlaceBoundary.get_nearest_to(self.location)
-                except PlaceBoundary.DoesNotExist:
-                    self._nearest_city = None
-            return self._nearest_city
+            cached = self.get_cached('nearest_city', PlaceBoundary)
+            if cached:
+                return cached
+            try:
+                result = PlaceBoundary.get_nearest_to(self.location)
+                self.set_cached('nearest_city', result)
+                return result
+            except PlaceBoundary.DoesNotExist:
+                pass
+        return None
 
     def __unicode__(self):
         return u"%s's location at %s" % (
